unit VIdle;

(*
 TVIdleTimer V1.01 - released January 25, 1998

 Copyright (c) 1998 by Verisoft AG, Switzerland.

 Author: Wolfgang Rohdewald

 This component contains partial code from the TTimeoutDatabase by Demian.
 Demian can be contacted at demian@bhnet.com.br

 Disclaimer:
 This component is distributed as Freeware. There is no charge or warranty
 whatsoever. The author will not be held responsible for any direct or indirect
 damage resulting from the use of the component or any derivatives thereof.
 In short: USE IT AT YOUR OWN RISK! You are allowed to include the component
 code in any commercial or non commercial product. You are also allowed to make
 as many copies of the component as you want and distribute them, as long as you
 do not receive any money for the copying/distribution.

 You can send any comments, suggestions and bugs to wr@poboxes.com.

 The 16bit code is untested yet. I will probably never again use Delphi 1.

 If you do any modification to this code, please send me a copy.
 If you use this code on any 'real-life' application, please let me know.
 If you think this code is completely useless, please DON'T LET ME KNOW!

 What it does:
 TVIdleTimer is a descendent of TTimer with new properties and methods for
 determining the idle time of your application's user.

 How to use it:
 1. Install the component;
 2. Drop a TVIdleTimer component onto your form ;
 3. Set the Timeout property;
 4. Add a TimeOut Event Handler. It can read the IdleTime property

 By using the procedures Reset or ResetGlobal, you can also use this component
 for watching long batches: They should call Reset/Resetglobal regularly. If
 they don't, they have a problem, and your OnTimer event can abort them.

*)
interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls;

type
	TVIdleTimer = class(TTimer)
	private
		{ Private-Deklarationen }
		FEnabled: Boolean;
		procedure SetEnabled(Value: Boolean);
		function GetLastActivity: TDateTime;
		procedure SetLastActivity(const t: TDateTime);
		function GetIdleTime: TDateTime;
		function GetIdleMinutes: integer;
	protected
		{ Protected-Deklarationen }
	public
		{ Public-Deklarationen }
		constructor Create(AOwner: TComponent);override;
		destructor Destroy;override;
		property LastActivity: TDateTime read GetLastActivity write SetLastActivity;
			// last user activity.
		property IdleTime: TDateTime read GetIdleTime;
			// time passed since last user activity
		property IdleMinutes: Integer read GetIdleMinutes;
			// minutes passed since last user activity
	published
		{ Published-Deklarationen }
		property Enabled: Boolean read FEnabled write SetEnabled default True;
			// Enabled is the same as TTimer.Enabled, but it also controls
			// monitoring user activity for ALL allocated TVIdleTimers. Use this
			// for temporary interruptions of monitoring.
	end;

procedure Register;

implementation

var
	GLastActivity: TDateTime;
	GInstanceCount: Integer;
	whKeyboard,whMouse: HHook;

procedure GReset;
begin
	GLastActivity := Now;
end;

{$IFDEF WIN32}
function MouseHookCallBack(Code: integer; Msg: WPARAM; MouseHook: LPARAM): LRESULT; stdcall;
{$ELSE}
function MouseHookCallBack(Code: integer; Msg: word; MouseHook: longint): longint; export;
{$ENDIF}
begin
	if Code >= 0 then GReset;
	Result := CallNextHookEx(whMouse,Code,Msg,MouseHook);
end;

{$IFDEF WIN32}
function KeyboardHookCallBack(Code: integer; Msg: WPARAM; KeyboardHook: LPARAM): LRESULT; stdcall;
{$ELSE}
function KeyboardHookCallBack(Code: integer; Msg: word; KeyboardHook: longint): longint; export;
{$ENDIF}
begin
	if Code >= 0 then GReset;
	Result := CallNextHookEx(whKeyboard,Code,Msg,KeyboardHook);
end;

function HookActive: Boolean;
begin
	Result := whKeyboard <> 0;
end;

procedure CreateHooks;
	function GetModuleHandleFromInstance: THandle;
	var
		s: array[0..512] of char;
	begin
		GetModuleFileName(hInstance,s,sizeof(s)-1);
		Result := GetModuleHandle(s);
	end;
begin
	if not HookActive then begin
		whMouse := SetWindowsHookEx(WH_MOUSE,MouseHookCallBack,
																GetModuleHandleFromInstance,
																{$IFDEF WIN32}GetCurrentThreadID{$ELSE}GetCurrentTask{$ENDIF});
		whKeyboard := SetWindowsHookEx(WH_KEYBOARD,KeyboardHookCallBack,
																	 GetModuleHandleFromInstance,
																	 {$IFDEF WIN32}GetCurrentThreadID{$ELSE}GetCurrentTask{$ENDIF});
	end;
end;

procedure RemoveHooks;
begin
	if HookActive then
		try
			UnhookWindowsHookEx(whKeyboard);
			UnhookWindowsHookEx(whMouse);
		finally
			whKeyboard := 0;
			whMouse := 0;
		end;
end;

constructor TVIdleTimer.Create;
begin
	inherited;
	CreateHooks;
	if GInstanceCount = 0 then GReset;
	Inc(GInstanceCount);
end;

destructor TVIdleTimer.Destroy;
begin
	Dec(GInstanceCount);
	if GInstanceCount <= 0 then RemoveHooks;
	inherited;
end;

procedure TVIdleTimer.SetEnabled(Value: Boolean);
begin
	if Value <> FEnabled then
	begin
		FEnabled := Value;
		inherited Enabled := Value;
		if FEnabled then
			CreateHooks
		else
			RemoveHooks
	end;
end;

function TVIdleTimer.GetLastActivity: TDateTime;
begin
	Result := GLastActivity;
end;

procedure TVIdleTimer.SetLastActivity(const t: TDateTime);
begin
	GLastActivity := t;
end;

function TVIdleTimer.GetIdleTime: TDateTime;
begin
	Result := Now - GLastActivity;
end;

function TVIdleTimer.GetIdleMinutes: Integer;
begin
	Result := Trunc(IdleTime*1440.1);
end;

procedure Register;
begin
	RegisterComponents('Verisoft', [TVIdleTimer]);
end;

initialization
	GInstanceCount := 0;
	whKeyboard := 0;
	whMouse := 0;
end.
